"""
Test that the SBWatchpoint::SetEnable API works.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbplatform, lldbplatformutil


class TestWatchpointSetEnable(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_disable_works(self):
        """Set a watchpoint, disable it, and make sure it doesn't get hit."""
        self.build()
        self.do_test(False)

    def test_disable_enable_works(self):
        """Set a watchpoint, disable it, and make sure it doesn't get hit."""
        self.build()
        self.do_test(True)

    def do_test(self, test_enable):
        """Set a watchpoint, disable it and make sure it doesn't get hit."""

        main_file_spec = lldb.SBFileSpec("main.c")

        self.target = self.createTestTarget()

        bkpt_before = self.target.BreakpointCreateBySourceRegex(
            "Set a breakpoint here", main_file_spec
        )
        self.assertEqual(
            bkpt_before.GetNumLocations(), 1, "Failed setting the before breakpoint."
        )

        bkpt_after = self.target.BreakpointCreateBySourceRegex(
            "We should have stopped", main_file_spec
        )
        self.assertEqual(
            bkpt_after.GetNumLocations(), 1, "Failed setting the after breakpoint."
        )

        process = self.target.LaunchSimple(
            None, None, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        thread = lldbutil.get_one_thread_stopped_at_breakpoint(process, bkpt_before)
        self.assertTrue(thread.IsValid(), "We didn't stop at the before breakpoint.")

        ret_val = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand(
            "watchpoint set variable -w write global_var", ret_val
        )
        self.assertTrue(
            ret_val.Succeeded(), "Watchpoint set variable did not return success."
        )

        wp = self.target.FindWatchpointByID(1)
        self.assertTrue(wp.IsValid(), "Didn't make a valid watchpoint.")
        self.assertNotEqual(
            wp.GetWatchAddress(), lldb.LLDB_INVALID_ADDRESS, "Watch address is invalid"
        )

        wp.SetEnabled(False)
        self.assertTrue(not wp.IsEnabled(), "The watchpoint thinks it is still enabled")

        process.Continue()

        stop_reason = thread.GetStopReason()

        self.assertStopReason(
            stop_reason, lldb.eStopReasonBreakpoint, "We didn't stop at our breakpoint."
        )

        if test_enable:
            wp.SetEnabled(True)
            self.assertTrue(
                wp.IsEnabled(), "The watchpoint thinks it is still disabled."
            )
            process.Continue()
            stop_reason = thread.GetStopReason()
            self.assertStopReason(
                stop_reason,
                lldb.eStopReasonWatchpoint,
                "We didn't stop at our watchpoint",
            )



""" Testing explicit symbol loading via target symbols add. """
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TargetSymbolsAddCommand(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.source = "main.c"

    @no_debug_info_test  # Prevent the genaration of the dwarf version of this test
    @skipUnlessPlatform(["linux"])
    def test_target_symbols_add(self):
        """Test that 'target symbols add' can load the symbols
        even if gnu.build-id and gnu_debuglink are not present in the module.
        Similar to test_add_dsym_mid_execution test for macos."""
        self.build()
        exe = self.getBuildArtifact("stripped.out")

        self.target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.target, VALID_TARGET)

        main_bp = self.target.BreakpointCreateByName("main", "stripped.out")
        self.assertTrue(main_bp, VALID_BREAKPOINT)

        self.process = self.target.LaunchSimple(
            None, None, self.get_process_working_directory()
        )
        self.assertTrue(self.process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.assertState(
            self.process.GetState(), lldb.eStateStopped, STOPPED_DUE_TO_BREAKPOINT
        )

        exe_module = self.target.GetModuleAtIndex(0)

        # Check that symbols are not loaded and main.c is not know to be
        # the source file.
        self.expect("frame select", substrs=["main.c"], matching=False)

        # Tell LLDB that a.out has symbols for stripped.out
        self.runCmd(
            "target symbols add -s %s %s" % (exe, self.getBuildArtifact("a.out"))
        )

        # Check that symbols are now loaded and main.c is in the output.
        self.expect("frame select", substrs=["main.c"])



import os
import shutil

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestClangModuleAppUpdate(TestBase):
    @add_test_categories(["gmodules"])
    def test_rebuild_app_modules_untouched(self):
        with open(self.getBuildArtifact("module.modulemap"), "w") as f:
            f.write(
                """
                    module Foo { header "f.h" }
                    """
            )
        with open(self.getBuildArtifact("f.h"), "w") as f:
            f.write(
                """
                    @import ObjectiveC;
                    @interface Foo : NSObject {
                       int i;
                    }
                    +(instancetype)init;
                    @end
                    """
            )

        mod_cache = self.getBuildArtifact("private-module-cache")
        import os

        if os.path.isdir(mod_cache):
            shutil.rmtree(mod_cache)
        self.build()
        self.assertTrue(os.path.isdir(mod_cache), "module cache exists")

        target, process, _, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.m")
        )
        bar = target.FindTypes("Bar").GetTypeAtIndex(0)
        foo = bar.GetDirectBaseClassAtIndex(0).GetType()
        self.assertEqual(foo.GetNumberOfFields(), 1)
        self.assertEqual(foo.GetFieldAtIndex(0).GetName(), "i")

        # Rebuild.
        process.Kill()
        os.remove(self.getBuildArtifact("main.o"))
        os.remove(self.getBuildArtifact("a.out"))
        self.build()

        # Reattach.
        target, process, _, _ = lldbutil.run_to_breakpoint_do_run(self, target, bkpt)
        bar = target.FindTypes("Bar").GetTypeAtIndex(0)
        foo = bar.GetDirectBaseClassAtIndex(0).GetType()
        self.assertEqual(foo.GetNumberOfFields(), 1)
        self.assertEqual(foo.GetFieldAtIndex(0).GetName(), "i")



"""
Test that the batched breakpoint step-over optimization activates when
multiple threads hit the same breakpoint. Verifies that the optimization
reduces breakpoint toggle operations compared to stepping one at a time.
"""

import os
import re

from lldbsuite.test.decorators import *
from lldbsuite.test.concurrent_base import ConcurrentEventsBase
from lldbsuite.test.lldbtest import TestBase


@skipIfWindows
class ConcurrentBatchedBreakpointStepOver(ConcurrentEventsBase):
    @skipIf(triple="^mips")
    @skipIf(archs=["aarch64"])
    def test(self):
        """Test that batched breakpoint step-over reduces breakpoint
        toggle operations when multiple threads hit the same breakpoint."""
        self.build()

        num_threads = 10

        # Enable logging to capture optimization messages and GDB packets.
        lldb_logfile = self.getBuildArtifact("lldb-log.txt")
        self.runCmd("log enable lldb step break -f {}".format(lldb_logfile))

        gdb_logfile = self.getBuildArtifact("gdb-remote-log.txt")
        self.runCmd("log enable gdb-remote packets -f {}".format(gdb_logfile))

        # Run with breakpoint threads.
        self.do_thread_actions(num_breakpoint_threads=num_threads)

        self.assertTrue(os.path.isfile(lldb_logfile), "lldb log file not found")
        with open(lldb_logfile, "r") as f:
            lldb_log = f.read()

        # Verify the optimization activated by looking for "Registered thread"
        # messages, which indicate threads were grouped for batching.
        registered_matches = re.findall(
            r"Registered thread 0x[0-9a-fA-F]+ stepping over "
            r"breakpoint at (0x[0-9a-fA-F]+)",
            lldb_log,
        )
        self.assertGreater(
            len(registered_matches),
            0,
            "Expected batched breakpoint step-over optimization to be "
            "used (no 'Registered thread' messages found in log).",
        )
        thread_bp_addr = registered_matches[0]

        # Verify all threads completed their step-over.
        completed_count = lldb_log.count("Completed step over breakpoint plan.")
        self.assertGreaterEqual(
            completed_count,
            num_threads,
            "Expected at least {} 'Completed step over breakpoint plan.' "
            "messages (one per thread), but got {}.".format(
                num_threads, completed_count
            ),
        )

        # Count z0/Z0 packets for the thread breakpoint address.
        # z0 = remove (disable) software breakpoint.
        # Z0 = set (enable) software breakpoint.
        # Strip the "0x" prefix and leading zeros to match the GDB packet
        # format (which uses lowercase hex without "0x" prefix).
        bp_addr_hex = thread_bp_addr[2:].lstrip("0") if thread_bp_addr else ""

        z0_count = 0  # disable packets
        Z0_count = 0  # enable packets
        initial_Z0_seen = False
        max_vcont_step_threads = 0  # largest number of s: actions in one vCont

        self.assertTrue(os.path.isfile(gdb_logfile), "gdb-remote log file not found")
        with open(gdb_logfile, "r") as f:
            for line in f:
                if "send packet: $" not in line:
                    continue

                # Match z0,<addr> (disable) or Z0,<addr> (enable).
                m = re.search(r"send packet: \$([Zz])0,([0-9a-fA-F]+),", line)
                if m and m.group(2) == bp_addr_hex:
                    if m.group(1) == "Z":
                        if not initial_Z0_seen:
                            initial_Z0_seen = True
                        else:
                            Z0_count += 1
                    else:
                        z0_count += 1

                # Count step actions in vCont packets to detect batching.
                # A batched vCont looks like: vCont;s:tid1;s:tid2;...
                vcont_m = re.search(r"send packet: \$vCont((?:;[^#]+)*)", line)
                if vcont_m:
                    actions = vcont_m.group(1)
                    step_count = len(re.findall(r";s:", actions))
                    if step_count > max_vcont_step_threads:
                        max_vcont_step_threads = step_count

        # With the optimization, fewer breakpoint toggles should occur.
        # Without optimization we'd see num_threads z0 and num_threads Z0.
        # With batching, even partial, we expect fewer toggles.
        self.assertLess(
            z0_count,
            num_threads,
            "Expected fewer than {} breakpoint disables (z0) due to "
            "batching, but got {}.".format(num_threads, z0_count),
        )
        self.assertLess(
            Z0_count,
            num_threads,
            "Expected fewer than {} breakpoint re-enables (Z0) due to "
            "batching, but got {}.".format(num_threads, Z0_count),
        )

        # Verify at least one batched vCont packet contained multiple
        # step actions, proving threads were stepped together.
        self.assertGreater(
            max_vcont_step_threads,
            1,
            "Expected at least one batched vCont packet with multiple "
            "step actions (s:), but the maximum was {}.".format(max_vcont_step_threads),
        )



"""
Test SBTarget Read Instruction.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TargetReadInstructionsFlavor(TestBase):
    @skipIfDarwin
    @skipIfWindows
    @skipIf(archs=no_match(["x86_64", "x86", "i386"]))
    def test_read_instructions_with_flavor(self):
        self.build()
        executable = self.getBuildArtifact("a.out")

        # create a target
        target = self.dbg.CreateTarget(executable)
        self.assertTrue(target.IsValid(), "target is not valid")

        functions = target.FindFunctions("test_add")
        self.assertEqual(len(functions), 1)
        test_add = functions[0]

        test_add_symbols = test_add.GetSymbol()
        self.assertTrue(
            test_add_symbols.IsValid(), "test_add function symbols is not valid"
        )

        expected_instructions = (("mov", "eax, edi"), ("add", "eax, esi"), ("ret", ""))
        test_add_insts = test_add_symbols.GetInstructions(target, "intel")
        # clang adds an extra nop instruction but gcc does not. It makes more sense
        # to check if it is at least 3
        self.assertLessEqual(len(expected_instructions), len(test_add_insts))

        # compares only the expected instructions
        for expected_instr, instr in zip(expected_instructions, test_add_insts):
            self.assertTrue(instr.IsValid(), "instruction is not valid")
            expected_mnemonic, expected_op_str = expected_instr
            self.assertEqual(instr.GetMnemonic(target), expected_mnemonic)
            self.assertEqual(instr.GetOperands(target), expected_op_str)



"""
Test that variables of type integer are displayed correctly.
"""

import AbstractBase

from lldbsuite.test.decorators import *


class IntegerTypesTestCase(AbstractBase.GenericTester):
    def test_int_type(self):
        """Test that int-type variables are displayed correctly."""
        self.build_and_run("int.cpp", ["int"])

    @skipUnlessDarwin
    def test_int_type_from_block(self):
        """Test that int-type variables are displayed correctly from a block."""
        self.build_and_run("int.cpp", ["int"])

    def test_unsigned_int_type(self):
        """Test that 'unsigned_int'-type variables are displayed correctly."""
        self.build_and_run("unsigned_int.cpp", ["unsigned", "int"])

    @skipUnlessDarwin
    def test_unsigned_int_type_from_block(self):
        """Test that 'unsigned int'-type variables are displayed correctly from a block."""
        self.build_and_run("unsigned_int.cpp", ["unsigned", "int"], bc=True)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestStructuredBinding(TestBase):
    @skipIf(oslist=["linux"], archs=["arm$"])
    @skipIf(compiler="clang", compiler_version=["<", "14.0"])
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        self.expect_expr(
            "a1",
            result_type="A",
            result_children=[
                ValueCheck(name="x", type="int"),
                ValueCheck(name="y", type="int"),
            ],
        )
        self.expect_expr("b1", result_type="char", result_value="'a'")
        self.expect_expr("c1", result_type="char", result_value="'b'")
        self.expect_expr("d1", result_type="short", result_value="50")
        self.expect_expr("e1", result_type="int", result_value="60")
        self.expect_expr("f1", result_type="char", result_value="'c'")

        self.expect_expr(
            "a2",
            result_type="A",
            result_children=[
                ValueCheck(name="x", type="int"),
                ValueCheck(name="y", type="int"),
            ],
        )
        self.expect_expr("b2", result_type="char", result_value="'a'")
        self.expect_expr("c2", result_type="char", result_value="'b'")
        self.expect_expr("d2", result_type="short", result_value="50")
        self.expect_expr("e2", result_type="int", result_value="60")
        self.expect_expr("f2", result_type="char", result_value="'c'")

        self.expect_expr(
            "a3",
            result_type="A",
            result_children=[
                ValueCheck(name="x", type="int"),
                ValueCheck(name="y", type="int"),
            ],
        )
        self.expect_expr("b3", result_type="char", result_value="'a'")
        self.expect_expr("c3", result_type="char", result_value="'b'")
        self.expect_expr("d3", result_type="short", result_value="50")
        self.expect_expr("e3", result_type="int", result_value="60")
        self.expect_expr("f3", result_type="char", result_value="'c'")

        self.expect_expr("carr_ref1", result_type="char", result_value="'a'")
        self.expect_expr("carr_ref2", result_type="char", result_value="'b'")
        self.expect_expr("carr_ref3", result_type="char", result_value="'c'")

        self.expect_expr("sarr_ref1", result_type="short", result_value="11")
        self.expect_expr("sarr_ref2", result_type="short", result_value="12")
        self.expect_expr("sarr_ref3", result_type="short", result_value="13")

        self.expect_expr("iarr_ref1", result_type="int", result_value="22")
        self.expect_expr("iarr_ref2", result_type="int", result_value="33")
        self.expect_expr("iarr_ref3", result_type="int", result_value="44")

        self.expect_expr("carr_rref1", result_type="char", result_value="'a'")
        self.expect_expr("carr_rref2", result_type="char", result_value="'b'")
        self.expect_expr("carr_rref3", result_type="char", result_value="'c'")

        self.expect_expr("sarr_rref1", result_type="short", result_value="11")
        self.expect_expr("sarr_rref2", result_type="short", result_value="12")
        self.expect_expr("sarr_rref3", result_type="short", result_value="13")

        self.expect_expr("iarr_rref1", result_type="int", result_value="22")
        self.expect_expr("iarr_rref2", result_type="int", result_value="33")
        self.expect_expr("iarr_rref3", result_type="int", result_value="44")

        self.expect_expr("carr_copy1", result_type="char", result_value="'a'")
        self.expect_expr("carr_copy2", result_type="char", result_value="'b'")
        self.expect_expr("carr_copy3", result_type="char", result_value="'c'")

        self.expect_expr("sarr_copy1", result_type="short", result_value="11")
        self.expect_expr("sarr_copy2", result_type="short", result_value="12")
        self.expect_expr("sarr_copy3", result_type="short", result_value="13")

        self.expect_expr("iarr_copy1", result_type="int", result_value="22")
        self.expect_expr("iarr_copy2", result_type="int", result_value="33")
        self.expect_expr("iarr_copy3", result_type="int", result_value="44")

        self.expect_expr("tx1", result_value="4")
        self.expect_expr("ty1", result_value="'z'")
        self.expect_expr("tz1", result_value="10")

        self.expect_expr("tx2", result_value="4")
        self.expect_expr("ty2", result_value="'z'")
        self.expect_expr("tz2", result_value="10")

        # Older versions of Clang marked structured binding variables
        # as artificial, and thus LLDB wouldn't display them.
        if self.expectedCompiler(["clang"]) and self.expectedCompilerVersion(
            [">=", "22.0"]
        ):
            self.expect(
                "frame variable",
                substrs=[
                    "tx1 =",
                    "ty1 =",
                    "tz1 =",
                    "tx2 =",
                    "ty2 =",
                    "tz2 =",
                    "mp1 =",
                    "mp2 =",
                ],
            )



import os
import tempfile
import unittest

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *

# To be safe and portable, Unix domain socket paths should be kept at or below
# 108 characters on Linux, and around 104 characters on macOS:
MAX_SOCKET_PATH_LENGTH = 104


class MCPUnixSocketCommandTestCase(TestBase):
    @skipIfWindows
    @skipIfRemote
    @no_debug_info_test
    def test_unix_socket(self):
        """
        Test if we can start an MCP protocol-server accepting unix sockets
        """

        temp_directory = tempfile.TemporaryDirectory()
        socket_file = os.path.join(temp_directory.name, "mcp.sock")

        if len(socket_file) >= MAX_SOCKET_PATH_LENGTH:
            self.skipTest(
                f"Socket path {socket_file} exceeds the {MAX_SOCKET_PATH_LENGTH} character limit"
            )

        self.expect(
            f"protocol-server start MCP accept://{socket_file}",
            startstr="MCP server started with connection listeners:",
            substrs=[f"unix-connect://{socket_file}"],
        )

        self.expect(
            "protocol-server get MCP",
            startstr="MCP server connection listeners:",
            substrs=[f"unix-connect://{socket_file}"],
        )

        self.runCmd("protocol-server stop MCP", check=False)
        self.expect(
            "protocol-server get MCP",
            error=True,
            substrs=["MCP server is not running"],
        )



"""
Test the printing of anonymous and named namespace variables.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class NamespaceBreakpointTestCase(TestBase):
    @expectedFailureAll(bugnumber="llvm.org/pr28548", compiler="gcc")
    @expectedFailureAll(oslist=["windows"])
    def test_breakpoints_func_auto(self):
        """Test that we can set breakpoints correctly by basename to find all functions whose basename is "func"."""
        self.build()

        names = ["func()", "func(int)", "A::B::func()", "A::func()", "A::func(int)"]

        # Create a target by the debugger.
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        module_list = lldb.SBFileSpecList()
        module_list.Append(lldb.SBFileSpec(exe, False))
        self.assertEqual(
            module_list[0].GetFilename(), lldb.SBFileSpec(exe, False).GetFilename()
        )
        self.assertEqual(
            module_list[-1].GetFilename(), lldb.SBFileSpec(exe, False).GetFilename()
        )
        cu_list = lldb.SBFileSpecList()
        # Set a breakpoint by name "func" which should pick up all functions
        # whose basename is "func"
        bp = target.BreakpointCreateByName(
            "func", lldb.eFunctionNameTypeAuto, module_list, cu_list
        )
        for bp_loc in bp:
            name = bp_loc.GetAddress().GetFunction().GetName()
            self.assertIn(
                name,
                names,
                "make sure breakpoint locations are correct for 'func' with eFunctionNameTypeAuto",
            )

    @expectedFailureAll(bugnumber="llvm.org/pr28548", compiler="gcc")
    def test_breakpoints_func_full(self):
        """Test that we can set breakpoints correctly by fullname to find all functions whose fully qualified name is "func"
        (no namespaces)."""
        self.build()

        names = ["func()", "func(int)"]

        # Create a target by the debugger.
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        module_list = lldb.SBFileSpecList()
        module_list.Append(lldb.SBFileSpec(exe, False))
        cu_list = lldb.SBFileSpecList()

        # Set a breakpoint by name "func" whose fullly qualified named matches "func" which
        # should pick up only functions whose basename is "func" and has no
        # containing context
        bp = target.BreakpointCreateByName(
            "func", lldb.eFunctionNameTypeFull, module_list, cu_list
        )
        for bp_loc in bp:
            name = bp_loc.GetAddress().GetFunction().GetName()
            self.assertIn(
                name,
                names,
                "make sure breakpoint locations are correct for 'func' with eFunctionNameTypeFull",
            )

    def test_breakpoints_a_func_full(self):
        """Test that we can set breakpoints correctly by fullname to find all functions whose fully qualified name is "A::func"."""
        self.build()

        names = ["A::func()", "A::func(int)"]

        # Create a target by the debugger.
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        module_list = lldb.SBFileSpecList()
        module_list.Append(lldb.SBFileSpec(exe, False))
        cu_list = lldb.SBFileSpecList()

        # Set a breakpoint by name "A::func" whose fullly qualified named matches "A::func" which
        # should pick up only functions whose basename is "func" and is
        # contained in the "A" namespace
        bp = target.BreakpointCreateByName(
            "A::func", lldb.eFunctionNameTypeFull, module_list, cu_list
        )
        for bp_loc in bp:
            name = bp_loc.GetAddress().GetFunction().GetName()
            self.assertIn(
                name,
                names,
                "make sure breakpoint locations are correct for 'A::func' with eFunctionNameTypeFull",
            )


class NamespaceTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers for declarations of namespace variables i and
        # j.
        self.line_var_i = line_number(
            "main.cpp", "// Find the line number for anonymous namespace variable i."
        )
        self.line_var_j = line_number(
            "main.cpp", "// Find the line number for named namespace variable j."
        )
        # And the line number to break at.
        self.line_break = line_number("main.cpp", "// Set break point at this line.")
        # Break inside do {} while and evaluate value
        self.line_break_ns1 = line_number("main.cpp", "// Evaluate ns1::value")
        self.line_break_ns2 = line_number("main.cpp", "// Evaluate ns2::value")

    def runToBkpt(self, command):
        self.runCmd(command, RUN_SUCCEEDED)
        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

    # rdar://problem/8668674
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24764")
    def test_with_run_command(self):
        """Test that anonymous and named namespace variables display correctly."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.cpp",
            self.line_break_ns1,
            num_expected_locations=1,
            loc_exact=True,
        )
        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.cpp",
            self.line_break_ns2,
            num_expected_locations=1,
            loc_exact=True,
        )
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line_break, num_expected_locations=1, loc_exact=True
        )

        self.runToBkpt("run")
        # Evaluate ns1::value
        self.expect_expr("value", result_value="100")

        self.runToBkpt("continue")
        # Evaluate ns2::value
        self.expect_expr("value", result_value="200")

        self.runToBkpt("continue")
        # On Mac OS X, gcc 4.2 emits the wrong debug info with respect to
        # types.
        slist = ["(int) a = 12", "anon_uint", "a_uint", "b_uint", "y_uint"]
        if self.platformIsDarwin() and self.getCompiler() in ["clang"]:
            slist = [
                "(int) a = 12",
                "::my_uint_t",
                "anon_uint = 0",
                "(A::uint_t) a_uint = 1",
                "(A::B::uint_t) b_uint = 2",
                "(Y::uint_t) y_uint = 3",
            ]

        # 'frame variable' displays the local variables with type information.
        self.expect("frame variable", VARIABLES_DISPLAYED_CORRECTLY, substrs=slist)

        # 'frame variable' with basename 'i' should work.
        self.expect(
            "frame variable --show-declaration --show-globals i",
            startstr="main.cpp:%d: (int) (anonymous namespace)::i = 3"
            % self.line_var_i,
        )
        # main.cpp:12: (int) (anonymous namespace)::i = 3

        # 'frame variable' with basename 'j' should work, too.
        self.expect(
            "frame variable --show-declaration --show-globals j",
            startstr="main.cpp:%d: (int) A::B::j = 4" % self.line_var_j,
        )
        # main.cpp:19: (int) A::B::j = 4

        # 'frame variable' should support address-of operator.
        self.runCmd("frame variable &i")

        # 'frame variable' with fully qualified name 'A::B::j' should work.
        self.expect(
            "frame variable A::B::j",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(int) A::B::j = 4",
            patterns=[" = 4"],
        )

        # So should the anonymous namespace case.
        self.expect(
            "frame variable '(anonymous namespace)::i'",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(int) (anonymous namespace)::i = 3",
            patterns=[" = 3"],
        )

        # Search for a type in an anonymous namespace, both with and without the
        # namespace prefix.
        self.expect("type lookup -- my_uint_t", substrs=["unsigned int"])
        self.expect("type lookup -- (anonymous namespace)::my_uint_t",
                    substrs=["unsigned int"])

        # rdar://problem/8660275
        # test/namespace: 'expression -- i+j' not working
        # This has been fixed.
        self.expect_expr("i + j", result_type="int", result_value="7")
        # (int) $2 = 7

        self.expect_expr("i", result_value="3")
        self.expect_expr("j", result_value="4")

        # rdar://problem/8668674
        # expression command with fully qualified namespace for a variable does
        # not work
        self.expect_expr("::i", result_value="3")
        self.expect_expr("A::B::j", result_value="4")

        # expression command with function in anonymous namespace
        self.expect_expr("myanonfunc(3)", result_value="6")

        # global namespace qualification with function in anonymous namespace
        self.expect_expr("myanonfunc(4)", result_value="8")

        self.expect(
            "expression myanonfunc",
            patterns=[r"\(anonymous namespace\)::myanonfunc\(int\)"],
        )

        self.expect(
            "expression variadic_sum",
            patterns=[r"\(anonymous namespace\)::variadic_sum\(int, ...\)"],
        )

        self.expect_expr("::B::Bar b; b.x()", result_type="int", result_value="42")
        self.expect_expr("A::B::Bar b; b.y()", result_type="int", result_value="137")
        self.expect_expr(
            "::NS1::NS2::Foo{}.bar() == -2 && ::NS2::Foo{}.bar() == -3",
            result_type="bool",
            result_value="true",
        )
        # FIXME: C++ unqualified namespace lookups currently not supported when instantiating types.
        self.expect_expr(
            "NS2::Foo{}.bar() == -3", result_type="bool", result_value="false"
        )
        self.expect_expr(
            "((::B::Bar*)&::B::bar)->x()", result_type="int", result_value="42"
        )

        self.expect_expr("InAnon1::var_in_anon", result_type="int", result_value="10")
        self.expect_expr(
            "InAnon1::InAnon2::var_in_anon", result_type="int", result_value="5"
        )
        self.expect_expr(
            "InAnon1::inline_ns::var_in_anon", result_type="int", result_value="15"
        )
        self.expect_expr(
            "InAnon1::inline_ns::InAnon2::var_in_anon",
            result_type="int",
            result_value="5",
        )



"""
Test calling std::String member functions.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExprCommandCallFunctionTestCase(TestBase):
    @expectedFailureAll(
        compiler="icc", bugnumber="llvm.org/pr14437, fails with ICC 13.1"
    )
    @skipIf(compiler="clang", compiler_version=["<", "9.0"])
    def test_with(self):
        """Test calling std::String member function."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        self.expect("expression str", substrs=["Hello world"])

        # Calling this function now succeeds, but we follow the typedef return type through to
        # const char *, and thus don't invoke the Summary formatter.

        # clang's libstdc++ on ios arm64 inlines std::string::c_str() always;
        # skip this part of the test.
        triple = self.dbg.GetSelectedPlatform().GetTriple()
        do_cstr_test = True
        if triple in [
            "arm64-apple-ios",
            "arm64e-apple-ios",
            "arm64-apple-tvos",
            "armv7k-apple-watchos",
            "arm64-apple-bridgeos",
            "arm64_32-apple-watchos",
        ]:
            do_cstr_test = False
        if do_cstr_test:
            self.expect("expression str.c_str()", substrs=["Hello world"])



# This implements the "diagnose-unwind" command, usually installed
# in the debug session like
#   command script import lldb.diagnose
# it is used when lldb's backtrace fails -- it collects and prints
# information about the stack frames, and tries an alternate unwind
# algorithm, that will help to understand why lldb's unwind algorithm
# did not succeed.

import optparse
import lldb
import re
import shlex

# Print the frame number, pc, frame pointer, module UUID and function name
# Returns the SBModule that contains the PC, if it could be found


def backtrace_print_frame(target, frame_num, addr, fp):
    process = target.GetProcess()
    addr_for_printing = addr
    addr_width = process.GetAddressByteSize() * 2
    if frame_num > 0:
        addr = addr - 1

    sbaddr = lldb.SBAddress()
    try:
        sbaddr.SetLoadAddress(addr, target)
        module_description = ""
        if sbaddr.GetModule():
            module_filename = ""
            module_uuid_str = sbaddr.GetModule().GetUUIDString()
            if module_uuid_str is None:
                module_uuid_str = ""
            if sbaddr.GetModule().GetFileSpec():
                module_filename = sbaddr.GetModule().GetFileSpec().GetFilename()
                if module_filename is None:
                    module_filename = ""
            if module_uuid_str != "" or module_filename != "":
                module_description = "%s %s" % (module_filename, module_uuid_str)
    except Exception:
        print(
            "%2d: pc==0x%-*x fp==0x%-*x"
            % (frame_num, addr_width, addr_for_printing, addr_width, fp)
        )
        return

    sym_ctx = target.ResolveSymbolContextForAddress(
        sbaddr, lldb.eSymbolContextEverything
    )
    if sym_ctx.IsValid() and sym_ctx.GetSymbol().IsValid():
        function_start = sym_ctx.GetSymbol().GetStartAddress().GetLoadAddress(target)
        offset = addr - function_start
        print(
            "%2d: pc==0x%-*x fp==0x%-*x %s %s + %d"
            % (
                frame_num,
                addr_width,
                addr_for_printing,
                addr_width,
                fp,
                module_description,
                sym_ctx.GetSymbol().GetName(),
                offset,
            )
        )
    else:
        print(
            "%2d: pc==0x%-*x fp==0x%-*x %s"
            % (
                frame_num,
                addr_width,
                addr_for_printing,
                addr_width,
                fp,
                module_description,
            )
        )
    return sbaddr.GetModule()


# A simple stack walk algorithm that follows the frame chain.
# Returns a two-element list; the first element is a list of modules
# seen and the second element is a list of addresses seen during the backtrace.


def simple_backtrace(debugger):
    target = debugger.GetSelectedTarget()
    process = target.GetProcess()
    cur_thread = process.GetSelectedThread()

    initial_fp = cur_thread.GetFrameAtIndex(0).GetFP()

    # If the pseudoreg "fp" isn't recognized, on arm hardcode to r7 which is
    # correct for Darwin programs.
    if initial_fp == lldb.LLDB_INVALID_ADDRESS and target.triple[0:3] == "arm":
        for reggroup in cur_thread.GetFrameAtIndex(1).registers:
            if reggroup.GetName() == "General Purpose Registers":
                for reg in reggroup:
                    if reg.GetName() == "r7":
                        initial_fp = int(reg.GetValue(), 16)

    module_list = []
    address_list = [cur_thread.GetFrameAtIndex(0).GetPC()]
    this_module = backtrace_print_frame(
        target, 0, cur_thread.GetFrameAtIndex(0).GetPC(), initial_fp
    )
    print_stack_frame(process, initial_fp)
    print("")
    if this_module is not None:
        module_list.append(this_module)
    if cur_thread.GetNumFrames() < 2:
        return [module_list, address_list]

    cur_fp = process.ReadPointerFromMemory(initial_fp, lldb.SBError())
    cur_pc = process.ReadPointerFromMemory(
        initial_fp + process.GetAddressByteSize(), lldb.SBError()
    )

    frame_num = 1

    while (
        cur_pc != 0
        and cur_fp != 0
        and cur_pc != lldb.LLDB_INVALID_ADDRESS
        and cur_fp != lldb.LLDB_INVALID_ADDRESS
    ):
        address_list.append(cur_pc)
        this_module = backtrace_print_frame(target, frame_num, cur_pc, cur_fp)
        print_stack_frame(process, cur_fp)
        print("")
        if this_module is not None:
            module_list.append(this_module)
        frame_num = frame_num + 1
        next_pc = 0
        next_fp = 0
        if (
            target.triple[0:6] == "x86_64"
            or target.triple[0:4] == "i386"
            or target.triple[0:3] == "arm"
        ):
            error = lldb.SBError()
            next_pc = process.ReadPointerFromMemory(
                cur_fp + process.GetAddressByteSize(), error
            )
            if not error.Success():
                next_pc = 0
            next_fp = process.ReadPointerFromMemory(cur_fp, error)
            if not error.Success():
                next_fp = 0
        # Clear the 0th bit for arm frames - this indicates it is a thumb frame
        if target.triple[0:3] == "arm" and (next_pc & 1) == 1:
            next_pc = next_pc & ~1
        cur_pc = next_pc
        cur_fp = next_fp
    this_module = backtrace_print_frame(target, frame_num, cur_pc, cur_fp)
    print_stack_frame(process, cur_fp)
    print("")
    if this_module is not None:
        module_list.append(this_module)
    return [module_list, address_list]


def print_stack_frame(process, fp):
    if fp == 0 or fp == lldb.LLDB_INVALID_ADDRESS or fp == 1:
        return
    addr_size = process.GetAddressByteSize()
    addr = fp - (2 * addr_size)
    i = 0
    outline = "Stack frame from $fp-%d: " % (2 * addr_size)
    error = lldb.SBError()
    try:
        while i < 5 and error.Success():
            address = process.ReadPointerFromMemory(addr + (i * addr_size), error)
            outline += " 0x%x" % address
            i += 1
        print(outline)
    except Exception:
        return


def diagnose_unwind(debugger, command, result, dict):
    """
    Gather diagnostic information to help debug incorrect unwind (backtrace)
    behavior in lldb.  When there is a backtrace that doesn't look
    correct, run this command with the correct thread selected and a
    large amount of diagnostic information will be printed, it is likely
    to be helpful when reporting the problem.
    """

    command_args = shlex.split(command)
    parser = create_diagnose_unwind_options()
    try:
        (options, args) = parser.parse_args(command_args)
    except:
        return
    target = debugger.GetSelectedTarget()
    if target:
        process = target.GetProcess()
        if process:
            thread = process.GetSelectedThread()
            if thread:
                lldb_versions_match = re.search(
                    r"[lL][lL][dD][bB]-(\d+)([.](\d+))?([.](\d+))?",
                    debugger.GetVersionString(),
                )
                lldb_version = 0
                lldb_minor = 0
                if (
                    len(lldb_versions_match.groups()) >= 1
                    and lldb_versions_match.groups()[0]
                ):
                    lldb_major = int(lldb_versions_match.groups()[0])
                if (
                    len(lldb_versions_match.groups()) >= 5
                    and lldb_versions_match.groups()[4]
                ):
                    lldb_minor = int(lldb_versions_match.groups()[4])

                modules_seen = []
                addresses_seen = []

                print("LLDB version %s" % debugger.GetVersionString())
                print("Unwind diagnostics for thread %d" % thread.GetIndexID())
                print("")
                print(
                    "============================================================================================="
                )
                print("")
                print("OS plugin setting:")
                debugger.HandleCommand(
                    "settings show target.process.python-os-plugin-path"
                )
                print("")
                print("Live register context:")
                thread.SetSelectedFrame(0)
                debugger.HandleCommand("register read")
                print("")
                print(
                    "============================================================================================="
                )
                print("")
                print("lldb's unwind algorithm:")
                print("")
                frame_num = 0
                for frame in thread.frames:
                    if not frame.IsInlined():
                        this_module = backtrace_print_frame(
                            target, frame_num, frame.GetPC(), frame.GetFP()
                        )
                        print_stack_frame(process, frame.GetFP())
                        print("")
                        if this_module is not None:
                            modules_seen.append(this_module)
                        addresses_seen.append(frame.GetPC())
                        frame_num = frame_num + 1
                print("")
                print(
                    "============================================================================================="
                )
                print("")
                print("Simple stack walk algorithm:")
                print("")
                (module_list, address_list) = simple_backtrace(debugger)
                if module_list and module_list is not None:
                    modules_seen += module_list
                if address_list and address_list is not None:
                    addresses_seen = set(addresses_seen)
                    addresses_seen.update(set(address_list))

                print("")
                print(
                    "============================================================================================="
                )
                print("")
                print("Modules seen in stack walks:")
                print("")
                modules_already_seen = set()
                for module in modules_seen:
                    if (
                        module is not None
                        and module.GetFileSpec().GetFilename() is not None
                    ):
                        if (
                            not module.GetFileSpec().GetFilename()
                            in modules_already_seen
                        ):
                            debugger.HandleCommand(
                                "image list %s" % module.GetFileSpec().GetFilename()
                            )
                            modules_already_seen.add(module.GetFileSpec().GetFilename())

                print("")
                print(
                    "============================================================================================="
                )
                print("")
                print("Disassembly ofaddresses seen in stack walks:")
                print("")
                additional_addresses_to_disassemble = addresses_seen
                for frame in thread.frames:
                    if not frame.IsInlined():
                        print(
                            "--------------------------------------------------------------------------------------"
                        )
                        print("")
                        print(
                            "Disassembly of %s, frame %d, address 0x%x"
                            % (
                                frame.GetFunctionName(),
                                frame.GetFrameID(),
                                frame.GetPC(),
                            )
                        )
                        print("")
                        if (
                            target.triple[0:6] == "x86_64"
                            or target.triple[0:4] == "i386"
                        ):
                            debugger.HandleCommand(
                                "disassemble -F att -a 0x%x" % frame.GetPC()
                            )
                        else:
                            debugger.HandleCommand(
                                "disassemble -a 0x%x" % frame.GetPC()
                            )
                        if frame.GetPC() in additional_addresses_to_disassemble:
                            additional_addresses_to_disassemble.remove(frame.GetPC())

                for address in list(additional_addresses_to_disassemble):
                    print(
                        "--------------------------------------------------------------------------------------"
                    )
                    print("")
                    print("Disassembly of 0x%x" % address)
                    print("")
                    if target.triple[0:6] == "x86_64" or target.triple[0:4] == "i386":
                        debugger.HandleCommand("disassemble -F att -a 0x%x" % address)
                    else:
                        debugger.HandleCommand("disassemble -a 0x%x" % address)

                print("")
                print(
                    "============================================================================================="
                )
                print("")
                additional_addresses_to_show_unwind = addresses_seen
                for frame in thread.frames:
                    if not frame.IsInlined():
                        print(
                            "--------------------------------------------------------------------------------------"
                        )
                        print("")
                        print(
                            "Unwind instructions for %s, frame %d"
                            % (frame.GetFunctionName(), frame.GetFrameID())
                        )
                        print("")
                        debugger.HandleCommand(
                            'image show-unwind -a "0x%x"' % frame.GetPC()
                        )
                        if frame.GetPC() in additional_addresses_to_show_unwind:
                            additional_addresses_to_show_unwind.remove(frame.GetPC())

                for address in list(additional_addresses_to_show_unwind):
                    print(
                        "--------------------------------------------------------------------------------------"
                    )
                    print("")
                    print("Unwind instructions for 0x%x" % address)
                    print("")
                    debugger.HandleCommand('image show-unwind -a "0x%x"' % address)


def create_diagnose_unwind_options():
    usage = "usage: %prog"
    description = """Print diagnostic information about a thread backtrace which will help to debug unwind problems"""
    parser = optparse.OptionParser(
        description=description, prog="diagnose_unwind", usage=usage
    )
    return parser


def __lldb_init_module(debugger, internal_dict):
    debugger.HandleCommand(
        "command script add -o -f %s.diagnose_unwind diagnose-unwind" % __name__
    )
    print(
        'The "diagnose-unwind" command has been installed, type "help diagnose-unwind" for detailed help.'
    )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

import json
import uuid
import os
import shutil
import time


class TestObjectFileJSON(TestBase):
    TRIPLE = "arm64-apple-macosx13.0.0"

    def setUp(self):
        TestBase.setUp(self)
        self.source = "main.c"

    def emitJSON(self, data, path):
        json_object = json.dumps(data, indent=4)
        with open(path, "w") as outfile:
            outfile.write(json_object)

    def toModuleSpec(self, path):
        module_spec = lldb.SBModuleSpec()
        module_spec.SetFileSpec(lldb.SBFileSpec(path))
        return module_spec

    @no_debug_info_test
    def test_target(self):
        triple = "arm64-apple-macosx13.0.0"
        data = {
            "triple": triple,
            "uuid": str(uuid.uuid4()),
            "type": "executable",
        }

        json_object_file = self.getBuildArtifact("a.json")
        self.emitJSON(data, json_object_file)

        target = self.dbg.CreateTarget(json_object_file)
        self.assertTrue(target.IsValid())
        self.assertEqual(target.GetTriple(), triple)

    @no_debug_info_test
    def test_module(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)

        data = {
            "triple": target.GetTriple(),
            "uuid": str(uuid.uuid4()),
        }

        json_object_file_b = self.getBuildArtifact("b.json")
        self.emitJSON(data, json_object_file_b)

        module = target.AddModule(self.toModuleSpec(json_object_file_b))
        self.assertFalse(module.IsValid())
        TEXT_file_addr = 0x100000000
        DATA_file_addr = 0x100001000
        foo_file_addr = TEXT_file_addr + 0x100
        bar_file_addr = DATA_file_addr + 0x10
        TEXT_size = 0x222
        text_size = 0x20
        DATA_size = 0x333
        foo_size = 0x11
        bar_size = 0x22
        slide = 0x100000000
        data = {
            "triple": target.GetTriple(),
            "uuid": str(uuid.uuid4()),
            "type": "sharedlibrary",
            "sections": [
                {
                    "user_id": 0x100,
                    "name": "__PAGEZERO",
                    "type": "container",
                    "address": 0,
                    "size": 0x100000000,
                    "flags": 0x101
              },
              {
                    "user_id": 0x200,
                    "name": "__TEXT",
                    "type": "container",
                    "address": TEXT_file_addr,
                    "size": TEXT_size,
                    "flags": 0x202,
                    "file_offset": 0,
                    "file_size": TEXT_size,
                    "read": True,
                    "write": False,
                    "execute": True,
                    "subsections": [
                        {
                            "name": "__text",
                            "type": "code",
                            "address": TEXT_file_addr,
                            "size": text_size,
                            "alignment": 2,
                            "read": True,
                            "write": False,
                            "execute": True,
                        },
                        {
                            "name": "__fake",
                            "address": TEXT_file_addr + 1 * text_size,
                            "size": text_size,
                            "fake": True
                        },
                        {
                            "name": "__encrypted",
                            "address": TEXT_file_addr + 2 * text_size,
                            "size": text_size,
                            "encrypted": True
                        },
                        {
                            "name": "__tls",
                            "address": TEXT_file_addr + 2 * text_size,
                            "size": text_size,
                            "thread_specific": True
                        }
                    ],
                },
                {
                    "name": "__DATA",
                    "type": "data",
                    "address": DATA_file_addr,
                    "size": DATA_size,
                    "read": True,
                    "write": True,
                    "execute": False,
                    "flags": 0x303,
                    "file_offset": DATA_file_addr - TEXT_file_addr,
                    "file_size": DATA_size,
                },
            ],
            "symbols": [
                {
                    "name": "foo",
                    "type": "code",
                    "address": foo_file_addr,
                    "size": foo_size,
                },
                {
                    "name": "bar",
                    "type": "data",
                    "address": bar_file_addr,
                    "size": bar_size,
                },
            ],
        }

        json_object_file_c = self.getBuildArtifact("c.json")
        self.emitJSON(data, json_object_file_c)

        module = target.AddModule(self.toModuleSpec(json_object_file_c))
        self.assertTrue(module.IsValid())

        TEXT_section = module.GetSectionAtIndex(0)
        self.assertTrue(TEXT_section.IsValid())
        self.assertEqual(TEXT_section.GetName(), "__PAGEZERO")
        self.assertEqual(TEXT_section.file_addr, 0)
        self.assertEqual(TEXT_section.size, 0x100000000)
        self.assertEqual(TEXT_section.GetSectionType(), lldb.eSectionTypeContainer)
        self.assertEqual(TEXT_section.GetNumSubSections(), 0)
        text_permissions = TEXT_section.GetPermissions()
        self.assertFalse((text_permissions & lldb.ePermissionsReadable) != 0)
        self.assertFalse((text_permissions & lldb.ePermissionsWritable) != 0)
        self.assertFalse((text_permissions & lldb.ePermissionsExecutable) != 0)

        TEXT_section = module.GetSectionAtIndex(1)
        self.assertTrue(TEXT_section.IsValid())
        self.assertEqual(TEXT_section.GetName(), "__TEXT")
        self.assertEqual(TEXT_section.file_addr, TEXT_file_addr)
        self.assertEqual(TEXT_section.size, TEXT_size)
        self.assertEqual(TEXT_section.file_offset, 0)
        self.assertEqual(TEXT_section.file_size, TEXT_size)
        self.assertEqual(TEXT_section.GetSectionType(), lldb.eSectionTypeContainer)
        self.assertEqual(TEXT_section.GetNumSubSections(), 4)
        text_permissions = TEXT_section.GetPermissions()
        self.assertTrue((text_permissions & lldb.ePermissionsReadable) != 0)
        self.assertFalse((text_permissions & lldb.ePermissionsWritable) != 0)
        self.assertTrue((text_permissions & lldb.ePermissionsExecutable) != 0)

        text_section = TEXT_section.GetSubSectionAtIndex(0)
        self.assertEqual(text_section.GetName(), "__text")
        self.assertEqual(text_section.size, text_size)
        self.assertEqual(text_section.GetAlignment(), 4)
        self.assertEqual(text_section.GetSectionType(), lldb.eSectionTypeCode)
        self.assertEqual(text_section.GetNumSubSections(), 0)
        text_permissions = text_section.GetPermissions()
        self.assertTrue((text_permissions & lldb.ePermissionsReadable) != 0)
        self.assertFalse((text_permissions & lldb.ePermissionsWritable) != 0)
        self.assertTrue((text_permissions & lldb.ePermissionsExecutable) != 0)

        DATA_section = module.GetSectionAtIndex(2)
        self.assertTrue(DATA_section.IsValid())
        self.assertEqual(DATA_section.GetName(), "__DATA")
        self.assertEqual(DATA_section.file_addr, DATA_file_addr)
        self.assertEqual(DATA_section.size, DATA_size)
        self.assertEqual(DATA_section.file_offset, DATA_file_addr - TEXT_file_addr)
        self.assertEqual(DATA_section.file_size, DATA_size)
        self.assertEqual(DATA_section.GetSectionType(), lldb.eSectionTypeData)
        data_permissions = DATA_section.GetPermissions()
        self.assertTrue((data_permissions & lldb.ePermissionsReadable) != 0)
        self.assertTrue((data_permissions & lldb.ePermissionsWritable) != 0)
        self.assertFalse((data_permissions & lldb.ePermissionsExecutable) != 0)

        foo_symbol = module.FindSymbol("foo")
        self.assertTrue(foo_symbol.IsValid())
        self.assertEqual(foo_symbol.addr.GetFileAddress(), foo_file_addr)
        self.assertEqual(foo_symbol.GetSize(), foo_size)

        bar_symbol = module.FindSymbol("bar")
        self.assertTrue(bar_symbol.IsValid())
        self.assertEqual(bar_symbol.addr.GetFileAddress(), bar_file_addr)
        self.assertEqual(bar_symbol.GetSize(), bar_size)

        # Verify the user_ids and flags are set correctly since there is no API
        # for this on lldb.SBSection
        self.expect("target modules dump sections c.json",
            substrs = [
                "0x0000000000000100 container              [0x0000000000000000-0x0000000100000000)  ---  0x00000000 0x00000000 0x00000101 c.json.__PAGEZERO",
                "0x0000000000000200 container              [0x0000000100000000-0x0000000100000222)  r-x  0x00000000 0x00000222 0x00000202 c.json.__TEXT",
                "0x0000000000000001 code                   [0x0000000100000000-0x0000000100000020)  r-x  0x00000000 0x00000000 0x00000000 c.json.__TEXT.__text",
                "0x0000000000000002 code                   [0x0000000100000020-0x0000000100000040)  ---  0x00000000 0x00000000 0x00000000 c.json.__TEXT.__fake",
                "0x0000000000000003 code                   [0x0000000100000040-0x0000000100000060)  ---  0x00000000 0x00000000 0x00000000 c.json.__TEXT.__encrypted",
                "0x0000000000000004 code                   [0x0000000100000040-0x0000000100000060)  ---  0x00000000 0x00000000 0x00000000 c.json.__TEXT.__tls",
                "0x0000000000000005 data                   [0x0000000100001000-0x0000000100001333)  rw-  0x00001000 0x00000333 0x00000303 c.json.__DATA"
            ])

        error = target.SetSectionLoadAddress(TEXT_section, TEXT_file_addr + slide)
        self.assertSuccess(error)
        error = target.SetSectionLoadAddress(DATA_section, DATA_file_addr + slide)
        self.assertSuccess(error)
        self.assertEqual(foo_symbol.addr.GetLoadAddress(target), foo_file_addr + slide)
        self.assertEqual(bar_symbol.addr.GetLoadAddress(target), bar_file_addr + slide)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestStringPrinter(TestBase):
    def test(self):
        self.build()

        self.addTearDownHook(
            lambda x: x.runCmd("setting set escape-non-printables true")
        )

        lldbutil.run_to_source_breakpoint(
            self, "Break here", lldb.SBFileSpec("main.cpp", False)
        )

        self.expect_var_path(
            "charwithtabs",
            summary='"Hello\\t\\tWorld\\nI am here\\t\\tto say hello\\n"',
        )
        self.expect_var_path("a.data", summary='"FOOB"')
        self.expect_var_path("b.data", summary=r'"FO\0B"')
        self.expect_var_path("c.data", summary=r'"F\0O"')
        self.expect_var_path("manytrailingnuls", summary=r'"F\0OO\0BA\0R"')

        for c in ["", "const"]:
            for v in ["", "volatile"]:
                for s in ["", "unsigned"]:
                    summary = '"' + c + v + s + 'char"'
                    self.expect_var_path(c + v + s + "chararray", summary=summary)
                    # These should be printed normally
                    self.expect_var_path(c + v + s + "charstar", summary=summary)

        Schar5 = self.expect_var_path(
            "Schar5", children=[ValueCheck(name="x", value="0")]
        )
        self.assertIsNone(Schar5.GetSummary())
        Scharstar = self.expect_var_path(
            "Scharstar", children=[ValueCheck(name="x", value="0")]
        )
        self.assertIsNone(Scharstar.GetSummary())

        self.runCmd("setting set escape-non-printables false")
        self.expect_var_path(
            "charwithtabs", summary='"Hello\t\tWorld\nI am here\t\tto say hello\n"'
        )
        self.assertTrue(
            self.frame().FindVariable("longconstcharstar").GetSummary().endswith('"...')
        )

        # FIXME: make "b.data" and "c.data" work sanely

        self.expect("frame variable ref", substrs=['(&ref = "Hello")'])
        self.expect_var_path(
            "ref",
            summary=None,
            children=[ValueCheck(name="&ref", summary='"Hello"')],
        )

        # FIXME: should LLDB use "&&refref" for the name here?
        self.expect("frame variable refref", substrs=['(&refref = "Hi")'])
        self.expect_var_path(
            "refref",
            summary=None,
            children=[ValueCheck(name="&refref", summary='"Hi"')],
        )



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TypeSummaryListArgumentTestCase(TestBase):
    @no_debug_info_test
    def test_type_summary_list_with_arg(self):
        """Test that the 'type summary list' command handles command line arguments properly"""
        self.expect(
            "type summary list Foo", substrs=["Category: default", "Category: system"]
        )
        self.expect(
            "type summary list char", substrs=[r"char ?(\*|\[\])", r"char ?\[[0-9]+\]"]
        )

        self.expect("type summary list -w default", substrs=["system"], matching=False)
        self.expect(
            "type summary list -w system unsigned",
            substrs=["default", "0-9"],
            matching=False,
        )
        self.expect(
            "type summary list -w system char",
            substrs=[r"char ?(\*|\[\])", r"char ?\[[0-9]+\]"],
            matching=True,
        )



"""
Test using SendAsyncInterrupt to interrupt an "attach wait"
"""

import lldb
import sys
import time
import threading
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil


class AttachCancelTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIf(
        remote=True,
        hostoslist=["windows", "linux"],
        bugnumber="https://github.com/llvm/llvm-project/issues/115618",
    )
    def test_scripted_implementation(self):
        """Test that cancelling a stuck "attach waitfor" works."""
        # First make an empty target for the attach:
        target = self.dbg.CreateTarget(None)

        # We need to cancel this, so we need to do the attach
        # on a separate thread:
        class AttachThread(threading.Thread):
            def __init__(self, target, error):
                # Make this a daemon thread so if we don't manage to interrupt,
                # Python will keep this thread from hanging the test.
                threading.Thread.__init__(self, daemon=True)
                self.target = target
                self.error = error

            def run(self):
                self.target.AttachToProcessWithName(
                    lldb.SBListener(), "LLDB-No-Such-Process", True, self.error
                )

        error = lldb.SBError()
        thread = AttachThread(target, error)
        thread.start()

        # Now wait till the attach on the child thread has made a process
        # for the attach attempt:
        while not target.process.IsValid():
            time.sleep(1)
        # I don't have a positive signal for "we started the attach attempt"
        # so the best I can do is sleep a bit more here to give that a chance
        # to start:
        time.sleep(1)

        # Now send the attach interrupt:
        target.process.SendAsyncInterrupt()
        # We don't want to stall if we can't interrupt, so join with a timeout:
        thread.join(60)
        if thread.is_alive():
            self.fail("The attach thread is alive after timeout interval")

        # Now check the error, should say the attach was interrupted:
        self.assertTrue(error.Fail(), "We succeeded in not attaching")



"""Test that types defined in shared libraries work correctly."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class SharedLibTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.source = "shared.c"
        self.line = line_number(self.source, "// Set breakpoint 0 here.")
        self.shlib_names = ["foo"]

    def common_setup(self):
        # Run in synchronous mode
        self.dbg.SetAsync(False)
        self.runCmd("settings set symbols.load-on-demand true")

        # Create a target by the debugger.
        self.target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(self.target, VALID_TARGET)

        # Register our shared libraries for remote targets so they get
        # automatically uploaded
        self.environment = self.registerSharedLibrariesWithTarget(
            self.target, self.shlib_names
        )

        self.shared_lib_name = self.platformContext.getFullLibName("foo")

    @skipIfWindows
    def test_source_line_breakpoint(self):
        self.build()
        self.common_setup()

        lldbutil.run_break_set_by_file_and_line(
            self, "foo.c", 4, num_expected_locations=1, loc_exact=True
        )

        # Now launch the process, and do not stop at entry point.
        process = self.target.LaunchSimple(
            None, self.environment, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )
        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        thread = process.GetSelectedThread()
        stack_frames = thread.frames
        self.assertGreater(len(stack_frames), 2)

        leaf_frame = stack_frames[0]
        self.assertEqual("foo.c", leaf_frame.GetLineEntry().GetFileSpec().GetFilename())
        self.assertEqual(4, leaf_frame.GetLineEntry().GetLine())

        parent_frame = stack_frames[1]
        self.assertEqual(
            "shared.c", parent_frame.GetLineEntry().GetFileSpec().GetFilename()
        )
        self.assertEqual(7, parent_frame.GetLineEntry().GetLine())

    @skipIfWindows
    def test_symbolic_breakpoint(self):
        self.build()
        self.common_setup()

        lldbutil.run_break_set_by_symbol(
            self, "foo", sym_exact=True, num_expected_locations=1
        )

        # Now launch the process, and do not stop at entry point.
        process = self.target.LaunchSimple(
            None, self.environment, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )
        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        thread = process.GetSelectedThread()
        stack_frames = thread.frames
        self.assertGreater(len(stack_frames), 2)

        leaf_frame = stack_frames[0]
        self.assertEqual("foo.c", leaf_frame.GetLineEntry().GetFileSpec().GetFilename())
        self.assertEqual(4, leaf_frame.GetLineEntry().GetLine())

        parent_frame = stack_frames[1]
        self.assertEqual(
            "shared.c", parent_frame.GetLineEntry().GetFileSpec().GetFilename()
        )
        self.assertEqual(7, parent_frame.GetLineEntry().GetLine())

    @skipIfWindows
    def test_global_variable_hydration(self):
        self.build()
        self.common_setup()

        lldbutil.run_break_set_by_file_and_line(
            self, self.source, self.line, num_expected_locations=1, loc_exact=True
        )

        # Now launch the process, and do not stop at entry point.
        process = self.target.LaunchSimple(
            None, self.environment, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        self.expect(
            "target variable --shlib a.out",
            "Breakpoint in a.out should have hydrated the debug info",
            substrs=["global_shared = 897"],
        )

        self.expect(
            "target variable --shlib " + self.shared_lib_name,
            "shared library should not have debug info by default",
            matching=False,
            substrs=["global_foo"],
        )

        self.expect(
            "target variable global_foo --shlib " + self.shared_lib_name,
            "Match global_foo in symbol table should hydrate debug info",
            matching=True,
            substrs=["global_foo = 321"],
        )



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import transform
from mlir.dialects.transform import bufferization
from mlir.dialects.bufferization import LayoutMapOption


def run(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            print("\nTEST:", f.__name__)
            f()
        print(module)
    return f


@run
def testEmptyTensorToAllocTensorOpCompact():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("tensor.empty"),
    )
    with InsertionPoint(sequence.body):
        bufferization.EmptyTensorToAllocTensorOp(sequence.bodyTarget)
        transform.YieldOp()
    # CHECK-LABEL: TEST: testEmptyTensorToAllocTensorOpCompact
    # CHECK: = transform.bufferization.empty_tensor_to_alloc_tensor
    # CHECK-SAME: (!transform.op<"tensor.empty">) -> !transform.op<"bufferization.alloc_tensor">


@run
def testEmptyTensorToAllocTensorOpTyped():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("tensor.empty"),
    )
    with InsertionPoint(sequence.body):
        bufferization.EmptyTensorToAllocTensorOp(
            transform.OperationType.get("bufferization.alloc_tensor"),
            sequence.bodyTarget,
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: testEmptyTensorToAllocTensorOpTyped
    # CHECK: = transform.bufferization.empty_tensor_to_alloc_tensor
    # CHECK-SAME: (!transform.op<"tensor.empty">) -> !transform.op<"bufferization.alloc_tensor">


@run
def testOneShotBufferizeOpCompact():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate, [], transform.AnyOpType.get()
    )
    with InsertionPoint(sequence.body):
        bufferization.OneShotBufferizeOp(sequence.bodyTarget)
        transform.YieldOp()
    # CHECK-LABEL: TEST: testOneShotBufferizeOpCompact
    # CHECK: = transform.bufferization.one_shot_bufferize
    # CHECK-SAME: (!transform.any_op) -> !transform.any_op


@run
def testOneShotBufferizeOpTyped():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate, [], transform.AnyOpType.get()
    )
    with InsertionPoint(sequence.body):
        bufferization.OneShotBufferizeOp(
            transform.OperationType.get("test.dummy"),
            sequence.bodyTarget,
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: testOneShotBufferizeOpTyped
    # CHECK: = transform.bufferization.one_shot_bufferize
    # CHECK-SAME: (!transform.any_op) -> !transform.op<"test.dummy">


@run
def testOneShotBufferizeOpAttributes():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate, [], transform.AnyOpType.get()
    )
    with InsertionPoint(sequence.body):
        bufferization.OneShotBufferizeOp(
            sequence.bodyTarget,
            allow_return_allocs_from_loops=True,
            allow_unknown_ops=True,
            bufferize_function_boundaries=True,
            function_boundary_type_conversion=LayoutMapOption.IdentityLayoutMap,
            memcpy_op="linalg.copy",
            print_conflicts=True,
            test_analysis_only=True,
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: testOneShotBufferizeOpAttributes
    # CHECK: = transform.bufferization.one_shot_bufferize
    # CHECK-SAME: layout{IdentityLayoutMap}
    # CHECK-SAME: allow_return_allocs_from_loops = true
    # CHECK-SAME: allow_unknown_ops = true
    # CHECK-SAME: bufferize_function_boundaries = true
    # CHECK-SAME: memcpy_op = "linalg.copy"
    # CHECK-SAME: print_conflicts = true
    # CHECK-SAME: test_analysis_only = true
    # CHECK-SAME: (!transform.any_op) -> !transform.any_op



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test(self):
        self.expect_expr(
            "int i; __typeof__(i) j = 1; j", result_type="typeof (i)", result_value="1"
        )
        self.expect_expr(
            "int i; typeof(i) j = 1; j", result_type="typeof (i)", result_value="1"
        )
        self.expect_expr(
            "int i; decltype(i) j = 1; j", result_type="decltype(i)", result_value="1"
        )



"""
Test that addresses of functions compiled for Arm Thumb include the Thumb mode
bit (bit 0 of the address) when resolved and used in expressions.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestThumbFunctionAddr(TestBase):
    SHARED_BUILD_TESTCASE = False

    def do_thumb_function_test(self, language):
        self.build(dictionary={"CFLAGS_EXTRAS": f"-x {language} -mthumb"})

        exe = self.getBuildArtifact("a.out")
        line = line_number("main.c", "// Set break point at this line.")
        self.runCmd("target create %s" % exe)
        bpid = lldbutil.run_break_set_by_file_and_line(self, "main.c", line)

        self.runCmd("run")
        self.assertIsNotNone(
            lldbutil.get_one_thread_stopped_at_breakpoint_id(self.process(), bpid),
            "Process is not stopped at breakpoint",
        )

        # The compiler set this, so the mode bit will be included here.
        a_function_addr_var = (
            self.thread().GetFrameAtIndex(0).FindVariable("a_function_addr")
        )
        self.assertTrue(a_function_addr_var.IsValid())
        a_function_addr = a_function_addr_var.GetValueAsUnsigned()
        self.assertTrue(a_function_addr & 1)

        self.expect("p/x a_function_addr", substrs=[f"0x{a_function_addr:08x}"])
        # If lldb did not pay attention to the mode bit this would SIGILL trying
        # to execute Thumb encodings in Arm mode.
        self.expect("expression -- a_function()", substrs=["= 123"])

        # We cannot call GetCallableLoadAdress via. the API, so we expect this
        # to not have the bit set as it's treating it as a non-function symbol.
        found_function = self.target().FindFunctions("a_function")[0]
        self.assertTrue(found_function.IsValid())
        found_function = found_function.GetFunction()
        self.assertTrue(found_function.IsValid())
        found_function_addr = found_function.GetStartAddress()
        a_function_load_addr = found_function_addr.GetLoadAddress(self.target())
        self.assertEqual(a_function_load_addr, a_function_addr & ~1)

        # image lookup should not include the mode bit.
        a_function_file_addr = found_function_addr.GetFileAddress()
        self.expect(
            "image lookup -n a_function", substrs=[f"0x{a_function_file_addr:08x}"]
        )

    # This test is run for C and C++ because the two will take different paths
    # trying to resolve the function's address.

    @skipIf(archs=no_match(["arm$"]))
    @skipIf(archs=["arm64"])
    def test_function_addr_c(self):
        self.do_thumb_function_test("c")

    @skipIf(archs=no_match(["arm$"]))
    @skipIf(archs=["arm64"])
    def test_function_addr_cpp(self):
        self.do_thumb_function_test("c++")
